#define F_CPU 14745600UL   

#include <avr/io.h>
#include <string.h>
#include <util/delay.h>
#include <stdio.h>


//*******************************************************************************************
//  MOJE UPRAVY
//-------------------------------------------------------------------------------------------
//
#define setb(port,pin)    port |= 1<<pin    //nastav bit
#define clrb(port,pin)    port &= ~(1<<pin) //nuluj bit
#define negb(port,pin)    port ^= 1<<pin    //neguj bit

//***********************************************************************************
unsigned char DALLAS_1=4; /// na jakym je portu 
unsigned int teplota1,pr;
float T;
char bufferT[50];


// =========================== ONE WIRE RESET ======================
// komunikace se senzory teploty pomoc technologi 1-wire
// restart komunikace

unsigned char ow_reset(void)
{
  unsigned char presence;

  setb(DDRE, DALLAS_1);					// pin 5 portu C jako vystupni
  setb(PORTE, DALLAS_1);					// pin 5 portu C na 1
  
  clrb(PORTE, DALLAS_1);  					// pin 5 portu C na 0
  _delay_us(480); 						// ekn 

  clrb(DDRE, DALLAS_1);					// pin 5 portu C jako vstupn
  setb(PORTE, DALLAS_1);					// pin 5 portu C = zapnu PULL UP
  
  _delay_us(70); 						// ekn

  presence = bit_is_set(PINE, DALLAS_1); 			// odezva senzoru teploty

  _delay_us(410); 						// ekn na konec timeslot
  
  return(presence); 						
}

// =========================== ONE WIRE READ BIT ==================
// ten bitu na 1-wire

unsigned char read_bit(void)
{
  unsigned char presence;

  setb(DDRE, DALLAS_1);					// pin 5 portu C jako vystupn
  setb(PORTE, DALLAS_1);					// pin 5 portu C na 1
   
  clrb(PORTE, DALLAS_1);  					// pin 5 portu C na 0
  _delay_us(6);						// ekn
  
  clrb(DDRE, DALLAS_1);					// pin 5 portu C jako vstupn
  setb(PORTE, DALLAS_1);					// pin 5 portu C na 1 
  
  _delay_us(9); 						// ekn

  presence = bit_is_set(PINE, DALLAS_1); 			// odezva senzoru teploty

  _delay_us(55); 						// ekn

  return(presence); 						
}

// =========================== ONE WIRE READ BYTE =================
// ten bytu na 1-wire

unsigned char read_byte(void)
{
  unsigned char i;
  unsigned char value = 0;
  
  // vyuit pedchozho ten bitu na 1-wire, tu jednotliv bity po sob 
  for (i=0;i<8;i++)
  {
    if(read_bit()) value|=0x01<<i; 			
    _delay_us(128);							
  }

  return(value);
}

// =========================== ONE WIRE WRITE BIT =================
// zpis bitu na 1-wire

void write_bit(char bitval)
{
  setb(DDRE, DALLAS_1);					// pin 5 portu C jako vstupn
  setb(PORTE, DALLAS_1);					// pin 5 portu C na 1
      
  if (bitval==1) 
  {
    clrb(PORTE, DALLAS_1);					// pin 5 portu C na 0
    _delay_us(6);						// ekn

    clrb(DDRE, DALLAS_1);					// pin 5 portu C jako vstupn
    setb(PORTE, DALLAS_1);					// pin 5 portu C = zapnu PULL-UP
	
    _delay_us(64);						// ekn
  }

  if (bitval==0) 
  {
    clrb(PORTE, DALLAS_1);					// pin 5 portu C na 0
    _delay_us(60);						// ekn

    clrb(DDRE, DALLAS_1);					// pin 5 portu C jako vstupn
    setb(PORTE, DALLAS_1);					// pin 5 portu C = zapnu PULL-UP
	
    _delay_us(10);						// ekn
  }

}											

// =========================== ONE WIRE WRITE BYTE ================
// zpis bytu na 1-wire

void write_byte(char val)
{
  unsigned char i;
  unsigned char temp;

  // vyuit pedchozho zpisu bytu na 1-wire, zapisuji jednotliv byty po sob 
  for (i=0; i<8; i++) 						
  {
    temp = val>>i; 							
    temp &= 0x01; 							
    write_bit(temp); 						
}
    _delay_us(128);
}

// ========================= READ TEMPERATURE======================
// zjist aktuln teplotu, zkontroluje senzor a odele informaci o teplot

unsigned int Read_Temperature(void)
{
  // pomocn promnn
  char get[10];
  char temp_lsb,temp_msb;
  int k;
  ow_reset();
  
  // komunikace se senzorem teploty Dallas DS18B20
  // pro zjitn aktuln teploty
  write_byte(0xCC); 						
  write_byte(0x44); 					
  
  _delay_us(128);
  
  ow_reset();
  write_byte(0xCC); 					
  write_byte(0xBE); 						

  for (k=0;k<9;k++)
  {
    get[k]=read_byte();
  };

  temp_msb = get[1]; 									
  temp_lsb = get[0]; 								

  if (temp_msb <= 7)
  {
    unsigned int teplota = temp_msb;

	teplota = (temp_msb<<8);
	teplota = teplota + temp_lsb;
	teplota = teplota * 6.25;

					
    
	// vrt hodnotu aktuln teploty
	return teplota;
  };
}




